@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-4  R Development Core Team
+ *  Copyright (C) 2000-7  R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,13 +13,11 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not,  a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
-#define NONAMELESSUNION
-#include <stdlib.h>
-#include <stdio.h>
+#include <stdlib.h> /* for exit */
 
 extern int rcmdfn (int cmdarg, int argc, char **argv); /* in rcmdfn.c */
 
