@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2002   The R Development Core Team.
+ *  Copyright (C) 2002-2006   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,10 +13,21 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#if !defined(atanh) && defined(HAVE_DECL_ATANH) && !HAVE_DECL_ATANH
+extern double atanh(double x);
+#endif
+
+/* do this first to get the right options for math.h */
+#include <R_ext/Arith.h>
+
 #include <R.h>
 #include "ts.h"
 
@@ -31,7 +42,7 @@
 /* y vector length n of observations
    Z vector length p for observation equation y_t = Za_t +  eps_t
    a vector length p of initial state
-   P p x p matrix for initial state uncertainty (contempareneous)
+   P p x p matrix for initial state uncertainty (contemparaneous)
    T  p x p transition matrix
    V  p x p = RQR'
    h = var(eps_t)
@@ -41,31 +52,39 @@
 
    No checking here!
  */
-SEXP 
+SEXP
 KalmanLike(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
-	   SEXP sV, SEXP sh, SEXP sPn, SEXP sUP, SEXP op)
+	   SEXP sV, SEXP sh, SEXP sPn, SEXP sUP, SEXP op, SEXP fast)
 {
     SEXP res, ans = R_NilValue, resid = R_NilValue, states = R_NilValue;
-    int n = LENGTH(sy), p = LENGTH(sa), lop = asLogical(op);
-    double *y = REAL(sy), *Z = REAL(sZ), *a = REAL(sa), *P = REAL(sP), 
-	*T = REAL(sT), *V = REAL(sV), h = asReal(sh), *Pnew = REAL(sPn);
+    int n, p, lop = asLogical(op);
+    double *y, *Z, *a, *P, *T, *V, h = asReal(sh), *Pnew;
     double sumlog = 0.0, ssq = 0, resid0, gain, tmp, *anew, *mm, *M;
     int i, j, k, l;
 
-    /* It would be better to check types before using LENGTH and REAL
-       on these, but should still work this way.  LT */
     if (TYPEOF(sy) != REALSXP || TYPEOF(sZ) != REALSXP ||
 	TYPEOF(sa) != REALSXP || TYPEOF(sP) != REALSXP ||
+	TYPEOF(sPn) != REALSXP ||
 	TYPEOF(sT) != REALSXP || TYPEOF(sV) != REALSXP)
-	error("invalid argument type");
+	error(_("invalid argument type"));
+    n = LENGTH(sy); p = LENGTH(sa);
+    y = REAL(sy); Z = REAL(sZ); T = REAL(sT); V = REAL(sV);
+
+    /* Avoid modifying arguments unless fast=TRUE */
+    if (!LOGICAL(fast)[0]){
+	    PROTECT(sP = duplicate(sP));
+	    PROTECT(sa = duplicate(sa));
+	    PROTECT(sPn = duplicate(sPn));
+    }
+    P = REAL(sP); a = REAL(sa); Pnew = REAL(sPn);
 
     anew = (double *) R_alloc(p, sizeof(double));
     M = (double *) R_alloc(p, sizeof(double));
     mm = (double *) R_alloc(p * p, sizeof(double));
     if(lop) {
 	PROTECT(ans = allocVector(VECSXP, 3));
-	SET_VECTOR_ELT(ans, 1, resid=allocVector(REALSXP, n));	
-	SET_VECTOR_ELT(ans, 2, states=allocMatrix(REALSXP, n, p));
+	SET_VECTOR_ELT(ans, 1, resid = allocVector(REALSXP, n));
+	SET_VECTOR_ELT(ans, 2, states = allocMatrix(REALSXP, n, p));
     }
     for (l = 0; l < n; l++) {
 	for (i = 0; i < p; i++) {
@@ -91,6 +110,8 @@ KalmanLike(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
 		}
 	}
 	if (!ISNAN(y[l])) {
+	    double *rr = NULL /* -Wall */;
+	    if(lop) rr = REAL(resid);
 	    resid0 = y[l];
 	    for (i = 0; i < p; i++)
 		resid0 -= Z[i] * anew[i];
@@ -103,7 +124,7 @@ KalmanLike(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
 		gain += Z[i] * M[i];
 	    }
 	    ssq += resid0 * resid0 / gain;
-	    if(lop) REAL(resid)[l] = resid0 / sqrt(gain);
+	    if(lop) rr[l] = resid0 / sqrt(gain);
 	    sumlog += log(gain);
 	    for (i = 0; i < p; i++)
 		a[i] = anew[i] + M[i] * resid0 / gain;
@@ -111,34 +132,43 @@ KalmanLike(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
 		for (j = 0; j < p; j++)
 		    P[i + j * p] = Pnew[i + j * p] - M[i] * M[j] / gain;
 	} else {
+	    double *rr = NULL /* -Wall */;
+	    if(lop) rr = REAL(resid);
 	    for (i = 0; i < p; i++)
 		a[i] = anew[i];
 	    for (i = 0; i < p * p; i++)
 		P[i] = Pnew[i];
-	    if(lop) REAL(resid)[l] = NA_REAL;
+	    if(lop) rr[l] = NA_REAL;
+	}
+	if(lop) {
+	    double *rs = REAL(states);
+	    for(j = 0; j < p; j++) rs[l + n*j] = a[j];
 	}
-	if(lop)
-	    for(j = 0; j < p; j++) REAL(states)[l + n*j] = a[j];
     }
+
     if(lop) {
-	SET_VECTOR_ELT(ans, 0, res=allocVector(REALSXP, 2));	
+	SET_VECTOR_ELT(ans, 0, res=allocVector(REALSXP, 2));
 	REAL(res)[0] = ssq; REAL(res)[1] = sumlog;
 	UNPROTECT(1);
+	if (!LOGICAL(fast)[0])
+	    UNPROTECT(3);
 	return ans;
     } else {
 	res = allocVector(REALSXP, 2);
 	REAL(res)[0] = ssq; REAL(res)[1] = sumlog;
+	if (!LOGICAL(fast)[0])
+	    UNPROTECT(3);
 	return res;
     }
 }
 
-SEXP 
+SEXP
 KalmanSmooth(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
 	     SEXP sV, SEXP sh, SEXP sPn, SEXP sUP)
 {
     SEXP ssa, ssP, ssPn, res, states = R_NilValue, sN;
     int n = LENGTH(sy), p = LENGTH(sa);
-    double *y = REAL(sy), *Z = REAL(sZ), *a, *P, 
+    double *y = REAL(sy), *Z = REAL(sZ), *a, *P,
 	*T = REAL(sT), *V = REAL(sV), h = asReal(sh), *Pnew;
     double resid0, gain, tmp, *anew, *mm, *M;
     double *at, *rt, *Pt, *gains, *resids, *Mt, *L, gn, *Nt;
@@ -150,14 +180,14 @@ KalmanSmooth(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
     if (TYPEOF(sy) != REALSXP || TYPEOF(sZ) != REALSXP ||
 	TYPEOF(sa) != REALSXP || TYPEOF(sP) != REALSXP ||
 	TYPEOF(sT) != REALSXP || TYPEOF(sV) != REALSXP)
-	error("invalid argument type");
+	error(_("invalid argument type"));
 
     PROTECT(ssa = duplicate(sa)); a = REAL(ssa);
     PROTECT(ssP = duplicate(sP)); P = REAL(ssP);
     PROTECT(ssPn = duplicate(sPn)); Pnew = REAL(ssPn);
-    
+
     PROTECT(res = allocVector(VECSXP, 2));
-    SET_VECTOR_ELT(res, 0, states = allocMatrix(REALSXP, n, p)); 
+    SET_VECTOR_ELT(res, 0, states = allocMatrix(REALSXP, n, p));
     at = REAL(states);
     SET_VECTOR_ELT(res, 1, sN = allocVector(REALSXP, n*p*p));
     Nt = REAL(sN);
@@ -233,19 +263,19 @@ KalmanSmooth(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
     for (l = n - 1; l >= 0; l--) {
 	if (!ISNAN(gains[l])) {
 	    gn = 1/gains[l];
-	    for (i = 0; i < p; i++) 
+	    for (i = 0; i < p; i++)
 		rt[l + n * i] = Z[i] * resids[l] * gn;
 	} else {
 	    for (i = 0; i < p; i++) rt[l + n * i] = 0.0;
 	    gn = 0.0;
 	}
-	
+
 	if (var) {
 	    for (i = 0; i < p; i++)
 		for (j = 0; j < p; j++)
 		    Nt[l + n*i + n*p*j] = Z[i] * Z[j] * gn;
 	}
-	
+
 	if (l < n - 1) {
 	    /* compute r_{t-1} */
 	    for (i = 0; i < p; i++)
@@ -271,20 +301,20 @@ KalmanSmooth(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
 			for (k = 0; k < p; k++)
 			    tmp += L[k + p * i] * Nt[l + 1 + n*k + n*p*j];
 			mm[i + p * j] = tmp;
-		    }    
+		    }
 		for (i = 0; i < p; i++)
 		    for (j = 0; j < p; j++) {
 			tmp = 0.0;
 			for (k = 0; k < p; k++)
 			    tmp += mm[i + p * k] * L[k + p * j];
 			Nt[l + n*i + n*p*j] += tmp;
-		    }    
+		    }
 	    }
 	}
 
 	for (i = 0; i < p; i++) {
 	    tmp = 0.0;
-	    for (j = 0; j < p; j++) 
+	    for (j = 0; j < p; j++)
 		tmp += Pt[l + n*i + n*p*j] * rt[l + n * j];
 	    at[l + n*i] += tmp;
 	}
@@ -310,9 +340,9 @@ KalmanSmooth(SEXP sy, SEXP sZ, SEXP sa, SEXP sP, SEXP sT,
     return res;
 }
 
-SEXP 
-KalmanFore(SEXP nahead, SEXP sZ, SEXP sa0, SEXP sP0, SEXP sT, SEXP sV, 
-	   SEXP sh)
+SEXP
+KalmanFore(SEXP nahead, SEXP sZ, SEXP sa0, SEXP sP0, SEXP sT, SEXP sV,
+	   SEXP sh, SEXP fast)
 {
     SEXP res, forecasts, se;
     int  n = asReal(nahead), p = LENGTH(sa0);
@@ -326,7 +356,7 @@ KalmanFore(SEXP nahead, SEXP sZ, SEXP sa0, SEXP sP0, SEXP sT, SEXP sV,
     if (TYPEOF(sZ) != REALSXP ||
 	TYPEOF(sa0) != REALSXP || TYPEOF(sP0) != REALSXP ||
 	TYPEOF(sT) != REALSXP || TYPEOF(sV) != REALSXP)
-	error("invalid argument type");
+	error(_("invalid argument type"));
 
     anew = (double *) R_alloc(p, sizeof(double));
     Pnew = (double *) R_alloc(p * p, sizeof(double));
@@ -334,7 +364,12 @@ KalmanFore(SEXP nahead, SEXP sZ, SEXP sa0, SEXP sP0, SEXP sT, SEXP sV,
     PROTECT(res = allocVector(VECSXP, 2));
     SET_VECTOR_ELT(res, 0, forecasts = allocVector(REALSXP, n));
     SET_VECTOR_ELT(res, 1, se = allocVector(REALSXP, n));
-
+    if (!LOGICAL(fast)[0]){
+	PROTECT(sa0=duplicate(sa0));
+	a=REAL(sa0);
+	PROTECT(sP0=duplicate(sP0));
+	P=REAL(sP0);
+    }
     for (l = 0; l < n; l++) {
 	fc = 0.0;
 	for (i = 0; i < p; i++) {
@@ -380,7 +415,7 @@ static void partrans(int p, double *raw, double *new)
     int j, k;
     double a, work[100];
 
-    if(p > 100) error("can only transform 100 pars in arima0");
+    if(p > 100) error(_("can only transform 100 pars in arima0"));
 
     /* Step one: map (-Inf, Inf) to (-1, 1) via tanh
        The parameters are now the pacf phi_{kk} */
@@ -397,7 +432,7 @@ static void partrans(int p, double *raw, double *new)
 
 SEXP ARIMA_undoPars(SEXP sin, SEXP sarma)
 {
-    int *arma = INTEGER(sarma), mp = arma[0], mq = arma[1], msp = arma[2], 
+    int *arma = INTEGER(sarma), mp = arma[0], mq = arma[1], msp = arma[2],
 	i, v, n = LENGTH(sin);
     double *params, *in = REAL(sin);
     SEXP res = allocVector(REALSXP, n);
@@ -414,7 +449,7 @@ SEXP ARIMA_undoPars(SEXP sin, SEXP sarma)
 SEXP ARIMA_transPars(SEXP sin, SEXP sarma, SEXP strans)
 {
     int *arma = INTEGER(sarma), trans = asLogical(strans);
-    int mp = arma[0], mq = arma[1], msp = arma[2], msq = arma[3], 
+    int mp = arma[0], mq = arma[1], msp = arma[2], msq = arma[3],
 	ns = arma[4], i, j, p = mp + ns * msp, q = mq + ns * msq, v;
     double *in = REAL(sin), *params = REAL(sin), *phi, *theta;
     SEXP res, sPhi, sTheta;
@@ -459,16 +494,12 @@ SEXP ARIMA_transPars(SEXP sin, SEXP sarma, SEXP strans)
 }
 
 
-#ifdef WIN32
-extern double atanh(double);
-#endif
-
 static void invpartrans(int p, double *phi, double *new)
 {
     int j, k;
     double a, work[100];
 
-    if(p > 100) error("can only transform 100 pars in arima0");
+    if(p > 100) error(_("can only transform 100 pars in arima0"));
 
     for(j = 0; j < p; j++) work[j] = new[j] = phi[j];
     /* Run the Durbin-Levinson recursions backwards
@@ -484,7 +515,7 @@ static void invpartrans(int p, double *phi, double *new)
 
 SEXP ARIMA_Invtrans(SEXP in, SEXP sarma)
 {
-    int *arma = INTEGER(sarma), mp = arma[0], mq = arma[1], msp = arma[2], 
+    int *arma = INTEGER(sarma), mp = arma[0], mq = arma[1], msp = arma[2],
 	i, v, n = LENGTH(in);
     SEXP y = allocVector(REALSXP, n);
     double *raw = REAL(in), *new = REAL(y);
@@ -499,7 +530,7 @@ SEXP ARIMA_Invtrans(SEXP in, SEXP sarma)
 #define eps 1e-3
 SEXP ARIMA_Gradtrans(SEXP in, SEXP sarma)
 {
-    int *arma = INTEGER(sarma), mp = arma[0], mq = arma[1], msp = arma[2], 
+    int *arma = INTEGER(sarma), mp = arma[0], mq = arma[1], msp = arma[2],
 	i, j, v, n = LENGTH(in);
     SEXP y = allocMatrix(REALSXP, n, n);
     double *raw = REAL(in), *A = REAL(y), w1[100], w2[100], w3[100];
@@ -524,7 +555,7 @@ SEXP ARIMA_Gradtrans(SEXP in, SEXP sarma)
 	for(i = 0; i < msp; i++) {
 	    w1[i] += eps;
 	    partrans(msp, w1, w3);
-	    for(j = 0; j < msp; j++) 
+	    for(j = 0; j < msp; j++)
 		A[i + v + (j+v)*n] = (w3[j] - w2[j])/eps;
 	    w1[i] -= eps;
 	}
@@ -533,12 +564,12 @@ SEXP ARIMA_Gradtrans(SEXP in, SEXP sarma)
 }
 
 
-SEXP 
+SEXP
 ARIMA_Like(SEXP sy, SEXP sPhi, SEXP sTheta, SEXP sDelta,
 	   SEXP sa, SEXP sP, SEXP sPn, SEXP sUP, SEXP giveResid)
 {
     SEXP res, nres, sResid = R_NilValue;
-    int  n = LENGTH(sy), rd = LENGTH(sa), p = LENGTH(sPhi), 
+    int  n = LENGTH(sy), rd = LENGTH(sa), p = LENGTH(sPhi),
 	q = LENGTH(sTheta), d = LENGTH(sDelta), r = rd - d;
     double *y = REAL(sy), *a = REAL(sa), *P = REAL(sP), *Pnew = REAL(sPn);
     double *phi = REAL(sPhi), *theta = REAL(sTheta), *delta = REAL(sDelta);
@@ -546,12 +577,16 @@ ARIMA_Like(SEXP sy, SEXP sPhi, SEXP sTheta, SEXP sDelta,
 	*M;
     int i, j, k, l, nu = 0;
     Rboolean useResid = asLogical(giveResid);
+    double *rsResid = NULL /* -Wall */;
 
     anew = (double *) R_alloc(rd, sizeof(double));
     M = (double *) R_alloc(rd, sizeof(double));
     if (d > 0) mm = (double *) R_alloc(rd * rd, sizeof(double));
 
-    if (useResid) PROTECT(sResid = allocVector(REALSXP, n));
+    if (useResid) {
+	PROTECT(sResid = allocVector(REALSXP, n));
+	rsResid = REAL(sResid);
+    }
 
     for (l = 0; l < n; l++) {
 	for (i = 0; i < r; i++) {
@@ -643,7 +678,7 @@ ARIMA_Like(SEXP sy, SEXP sPhi, SEXP sTheta, SEXP sDelta,
 		ssq += resid * resid / gain;
 		sumlog += log(gain);
 	    }
-	    if (useResid) REAL(sResid)[l] = resid / sqrt(gain);
+	    if (useResid) rsResid[l] = resid / sqrt(gain);
 	    for (i = 0; i < rd; i++)
 		a[i] = anew[i] + M[i] * resid / gain;
 	    for (i = 0; i < rd; i++)
@@ -652,7 +687,7 @@ ARIMA_Like(SEXP sy, SEXP sPhi, SEXP sTheta, SEXP sDelta,
 	} else {
 	    for (i = 0; i < rd; i++) a[i] = anew[i];
 	    for (i = 0; i < rd * rd; i++) P[i] = Pnew[i];
-	    if (useResid) REAL(sResid)[l] = NA_REAL;
+	    if (useResid) rsResid[l] = NA_REAL;
 	}
     }
 
@@ -676,14 +711,14 @@ ARIMA_Like(SEXP sy, SEXP sPhi, SEXP sTheta, SEXP sDelta,
 
 /* do differencing here */
 /* arma is p, q, sp, sq, ns, d, sd */
-SEXP 
+SEXP
 ARIMA_CSS(SEXP sy, SEXP sarma, SEXP sPhi, SEXP sTheta,
 	  SEXP sncond, SEXP giveResid)
 {
-    SEXP res, nres, sResid = R_NilValue;
+    SEXP res, sResid = R_NilValue;
     double ssq = 0.0, *y = REAL(sy), tmp;
     double *phi = REAL(sPhi), *theta = REAL(sTheta), *w, *resid;
-    int n = LENGTH(sy), *arma = INTEGER(sarma), p = LENGTH(sPhi), 
+    int n = LENGTH(sy), *arma = INTEGER(sarma), p = LENGTH(sPhi),
 	q = LENGTH(sTheta), ncond = asInteger(sncond);
     int l, i, j, ns, nu = 0;
     Rboolean useResid = asLogical(giveResid);
@@ -713,16 +748,13 @@ ARIMA_CSS(SEXP sy, SEXP sarma, SEXP sPhi, SEXP sTheta,
     }
     if (useResid) {
 	PROTECT(res = allocVector(VECSXP, 2));
-	SET_VECTOR_ELT(res, 0, nres = allocVector(REALSXP, 1));
-	REAL(nres)[0] = ssq / (double) (nu);
+	SET_VECTOR_ELT(res, 0, ScalarReal(ssq / (double) (nu)));
 	SET_VECTOR_ELT(res, 1, sResid);
 	UNPROTECT(2);
 	return res;
     } else {
-	nres = allocVector(REALSXP, 1);
-	REAL(nres)[0] = ssq / (double) (nu);
 	UNPROTECT(1);
-	return nres;
+	return ScalarReal(ssq / (double) (nu));
     }
 }
 
@@ -730,6 +762,7 @@ SEXP TSconv(SEXP a, SEXP b)
 {
     int i, j, na, nb, nab;
     SEXP ab;
+    double *ra, *rb, *rab;
 
     PROTECT(a = coerceVector(a, REALSXP));
     PROTECT(b = coerceVector(b, REALSXP));
@@ -737,11 +770,11 @@ SEXP TSconv(SEXP a, SEXP b)
     nb = LENGTH(b);
     nab = na + nb - 1;
     PROTECT(ab = allocVector(REALSXP, nab));
-    for (i = 0; i < nab; i++)
-	REAL(ab)[i] = 0.0;
+    ra = REAL(a); rb = REAL(b); rab = REAL(ab);
+    for (i = 0; i < nab; i++) rab[i] = 0.0;
     for (i = 0; i < na; i++)
 	for (j = 0; j < nb; j++)
-	    REAL(ab)[i + j] += REAL(a)[i] * REAL(b)[j];
+	    rab[i + j] += ra[i] * rb[j];
     UNPROTECT(3);
     return (ab);
 }
@@ -791,12 +824,18 @@ SEXP getQ0(SEXP sPhi, SEXP sTheta)
 
     /* thetab[np], xnext[np], xrow[np].  rbar[rbar] */
     double *P, *xnext, *xrow, *rbar, *thetab;
-    int r = max(p, q + 1), np = r * (r + 1) / 2, nrbar = np * (np - 1) / 2;
+    /* NB: nrbar could overflow */
+    int r = max(p, q + 1);
+    size_t np = r * (r + 1) / 2, nrbar = np * (np - 1) / 2;
     int indi, indj, indn;
     double phii, phij, ynext, bi, vi, vj;
     int   i, j, ithisr, ind, npr, ind1, ind2, npr1, im, jm;
 
 
+    /* This is the limit using an int index.  We could use
+       size_t and get more on a 64-bit system, 
+       but there seems no practical need. */
+    if(r > 350) error(_("maximum supported lag is 350"));
     thetab = (double *) R_alloc(np, sizeof(double));
     xnext = (double *) R_alloc(np, sizeof(double));
     xrow = (double *) R_alloc(np, sizeof(double));
@@ -822,9 +861,9 @@ SEXP getQ0(SEXP sPhi, SEXP sTheta)
     }
     if (p > 0) {
 /*      The set of equations s * vec(P0) = vec(v) is solved for
-        vec(P0).  s is generated row by row in the array xnext.  The
-        order of elements in P is changed, so as to bring more leading
-        zeros into the rows of s. */
+	vec(P0).  s is generated row by row in the array xnext.  The
+	order of elements in P is changed, so as to bring more leading
+	zeros into the rows of s. */
 
 	for (i = 0; i < nrbar; i++) rbar[i] = 0.0;
 	for (i = 0; i < np; i++) {
