@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1998  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1999-2007  The R Development Core Team.
+ *  Copyright (C) 1999-2011  The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole string,
-   except that there is an assumption that filesep is one byte.
-*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -33,48 +30,28 @@
 
 static R_len_t asVecSize(SEXP x)
 {
-    int warn = 0, res;
-    double d;
-
     if (isVectorAtomic(x) && LENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
-	case LGLSXP:
-	    res = IntegerFromLogical(LOGICAL(x)[0], &warn);
-	    if(res == NA_INTEGER) error(_("vector size cannot be NA"));
-	    return res;
-	case INTSXP:
-	    res = INTEGER(x)[0];
+	case INTSXP: 
+	{
+	    int res = INTEGER(x)[0];
 	    if(res == NA_INTEGER) error(_("vector size cannot be NA"));
 	    return res;
+	}
 	case REALSXP:
-	    d = REAL(x)[0];
+	{
+	    double d = REAL(x)[0];
 	    if(ISNAN(d)) error(_("vector size cannot be NA/NaN"));
 	    if(!R_FINITE(d)) error(_("vector size cannot be infinite"));
 	    if(d < 0) error(_("vector size cannot be negative"));
 	    if(d > R_LEN_T_MAX) error(_("vector size specified is too large"));
 	    return (R_size_t) d;
 	}
+	}
     }
-    return -1;
+    return -1;  /* which gives error in the caller */
 }
 
-#ifdef UNUSED
-SEXP attribute_hidden do_delay(SEXP call, SEXP op, SEXP args, SEXP rho)
-{
-    SEXP expr, env;
-    checkArity(op, args);
-    expr = CAR(args);
-    env = eval(CADR(args), rho);
-    if (isNull(env)) {
-	error(_("use of NULL environment is defunct"));
-	env = R_BaseEnv;
-    } else
-    if (!isEnvironment(env))
-	errorcall(call, R_MSG_IA);
-    return mkPROMISE(expr, env);
-}
-#endif
-
 SEXP attribute_hidden do_delayed(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP name = R_NilValue /* -Wall */, expr, eenv, aenv;
@@ -94,7 +71,7 @@ SEXP attribute_hidden do_delayed(SEXP call, SEXP op, SEXP args, SEXP rho)
 	eenv = R_BaseEnv;
     } else
     if (!isEnvironment(eenv))
-	errorcall(call, R_MSG_IA);
+	errorcall(call, _("invalid '%s' argument"), "eval.env");
 
     args = CDR(args);
     aenv = CAR(args);
@@ -103,7 +80,7 @@ SEXP attribute_hidden do_delayed(SEXP call, SEXP op, SEXP args, SEXP rho)
 	aenv = R_BaseEnv;
     } else
     if (!isEnvironment(aenv))
-	errorcall(call, R_MSG_IA);
+	errorcall(call, _("invalid '%s' argument"), "assign.env");
 
     defineVar(name, mkPROMISE(expr, eenv), aenv);
     return R_NilValue;
@@ -122,9 +99,9 @@ SEXP attribute_hidden do_makelazy(SEXP call, SEXP op, SEXP args, SEXP rho)
     values = CAR(args); args = CDR(args);
     expr = CAR(args); args = CDR(args);
     eenv = CAR(args); args = CDR(args);
-    if (!isEnvironment(eenv)) error(R_MSG_IA);
+    if (!isEnvironment(eenv)) error(_("invalid '%s' argument"), "eval.env");
     aenv = CAR(args);
-    if (!isEnvironment(aenv)) error(R_MSG_IA);
+    if (!isEnvironment(aenv)) error(_("invalid '%s' argument"), "assign.env");
 
     for(i = 0; i < LENGTH(names); i++) {
 	SEXP name = install(CHAR(STRING_ELT(names, i)));
@@ -137,32 +114,32 @@ SEXP attribute_hidden do_makelazy(SEXP call, SEXP op, SEXP args, SEXP rho)
     return R_NilValue;
 }
 
+/* This is a primitive SPECIALSXP */
 SEXP attribute_hidden do_onexit(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     RCNTXT *ctxt;
-    SEXP code, add, oldcode, tmp;
+    SEXP code, oldcode, tmp, ap, argList;
     int addit = 0;
 
-    switch (length(args)) {
-    case 0:
-	code = R_NilValue;
-	break;
-    case 1:
-	code = CAR(args);
-	break;
-    case 2:
-	code = CAR(args);
-	add = eval(CADR(args), rho);
-	if ( TYPEOF(add) != LGLSXP || length(add) != 1 ||
-	     LOGICAL(add)[0] == NA_INTEGER)
+    PROTECT(ap = list2(R_NilValue, R_NilValue));
+    SET_TAG(ap,  install("expr"));
+    SET_TAG(CDR(ap), install("add"));
+    PROTECT(argList =  matchArgs(ap, args, call));
+    if (CAR(argList) == R_MissingArg) code = R_NilValue;
+    else code = CAR(argList);
+    if (CADR(argList) != R_MissingArg) {
+	addit = asLogical(eval(CADR(args), rho));
+	if (addit == NA_INTEGER)
 	    errorcall(call, _("invalid '%s' argument"), "add");
-	addit = (LOGICAL(add)[0] == 1);
-	break;
-    default:
-	errorcall_return(call, _("invalid number of arguments"));
     }
+
     ctxt = R_GlobalContext;
-    while (ctxt != R_ToplevelContext && !(ctxt->callflag & CTXT_FUNCTION) )
+    /* Search for the context to which the on.exit action is to be
+       attached. Lexical scoping is implemented by searching for the
+       first closure call context with an environment matching the
+       expression evaluation environment. */
+    while (ctxt != R_ToplevelContext &&
+	   !((ctxt->callflag & CTXT_FUNCTION) && ctxt->cloenv == rho) )
 	ctxt = ctxt->nextcontext;
     if (ctxt->callflag & CTXT_FUNCTION)
     {
@@ -179,7 +156,7 @@ SEXP attribute_hidden do_onexit(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    }
 	    else
 	    {
-		PROTECT(tmp=allocList(1));
+		PROTECT(tmp = allocList(1));
 		SETCAR(tmp, code);
 		ctxt->conexit = listAppend(duplicate(oldcode),tmp);
 		UNPROTECT(1);
@@ -188,6 +165,7 @@ SEXP attribute_hidden do_onexit(SEXP call, SEXP op, SEXP args, SEXP rho)
 	else
 	    ctxt->conexit = code;
     }
+    UNPROTECT(2);
     return R_NilValue;
 }
 
@@ -270,6 +248,10 @@ SEXP attribute_hidden do_bodyCode(SEXP call, SEXP op, SEXP args, SEXP rho)
     else return R_NilValue;
 }
 
+/* get environment from a subclass if possible; else return NULL */
+#define simple_as_environment(arg) (IS_S4_OBJECT(arg) && (TYPEOF(arg) == S4SXP) ? R_getS4DataSlot(arg, ENVSXP) : arg)
+
+
 SEXP attribute_hidden do_envir(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
@@ -283,22 +265,30 @@ SEXP attribute_hidden do_envir(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_envirgets(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP env, s = CAR(args);
+
     checkArity(op, args);
+    check1arg(args, call, "x");
+
     env = CADR(args);
 
     if (TYPEOF(CAR(args)) == CLOSXP
-	&& (isEnvironment(env) || isNull(env))) {
+	&& (isEnvironment(env) || 
+	    isEnvironment(env = simple_as_environment(env)) ||
+	    isNull(env))) {
 	if (isNull(env))
 	    error(_("use of NULL environment is defunct"));
-	if(NAMED(s) > 1) {
-	    /* partial duplicate */
-	    s = allocSExp(CLOSXP);
-	    SET_FORMALS(s, FORMALS(CAR(args)));
-	    SET_BODY(s, BODY(CAR(args)));
-	}
+	if(NAMED(s) > 1)
+	    /* this copies but does not duplicate args or code */
+	    s = duplicate(s);
+#ifdef BYTECODE
+	if (TYPEOF(BODY(s)) == BCODESXP)
+	    /* switch to interpreted version if compiled */
+	    SET_BODY(s, R_ClosureExpr(CAR(args)));
+#endif
 	SET_CLOENV(s, env);
     }
-    else if (isNull(env) || isEnvironment(env))
+    else if (isNull(env) || isEnvironment(env) ||
+	isEnvironment(env = simple_as_environment(env)))
 	setAttrib(s, R_DotEnvSymbol, env);
     else
 	error(_("replacement object is not an environment"));
@@ -306,6 +296,10 @@ SEXP attribute_hidden do_envirgets(SEXP call, SEXP op, SEXP args, SEXP rho)
 }
 
 
+/** do_newenv() :  .Internal(new.env(hash, parent, size))
+ *
+ * @return a newly created environment()
+ */
 SEXP attribute_hidden do_newenv(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP enclos, size, ans;
@@ -320,31 +314,33 @@ SEXP attribute_hidden do_newenv(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("use of NULL environment is defunct"));
 	enclos = R_BaseEnv;
     } else
-    if( !isEnvironment(enclos) )
+    if( !isEnvironment(enclos)   &&
+	!isEnvironment((enclos = simple_as_environment(enclos))))
 	error(_("'enclos' must be an environment"));
 
     if( hash ) {
 	args = CDR(args);
 	PROTECT(size = coerceVector(CAR(args), INTSXP));
-	if (INTEGER(size)[0] == NA_INTEGER || INTEGER(size)[0] <= 0)
-	    error(_("'size' must be a positive integer"));
+	if (INTEGER(size)[0] == NA_INTEGER)
+	    INTEGER(size)[0] = 0; /* so it will use the internal default */
 	ans = R_NewHashedEnv(enclos, size);
 	UNPROTECT(1);
     } else
 	ans = NewEnvironment(R_NilValue, R_NilValue, enclos);
     return ans;
-
 }
 
 SEXP attribute_hidden do_parentenv(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
+    SEXP arg = CAR(args);
 
-    if( !isEnvironment(CAR(args)) )
+    if( !isEnvironment(arg)  &&
+	!isEnvironment((arg = simple_as_environment(arg))))
 	error( _("argument is not an environment"));
-    if( CAR(args) == R_EmptyEnv )
+    if( arg == R_EmptyEnv )
 	error(_("the empty environment has no parent"));
-    return( ENCLOS(CAR(args)) );
+    return( ENCLOS(arg) );
 }
 
 SEXP attribute_hidden do_parentenvgets(SEXP call, SEXP op, SEXP args, SEXP rho)
@@ -357,7 +353,8 @@ SEXP attribute_hidden do_parentenvgets(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("use of NULL environment is defunct"));
 	env = R_BaseEnv;
     } else
-    if( !isEnvironment(env) )
+    if( !isEnvironment(env) &&
+	!isEnvironment((env = simple_as_environment(env))))
 	error(_("argument is not an environment"));
     if( env == R_EmptyEnv )
 	error(_("can not set parent of the empty environment"));
@@ -366,12 +363,13 @@ SEXP attribute_hidden do_parentenvgets(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("use of NULL environment is defunct"));
 	parent = R_BaseEnv;
     } else
-    if( !isEnvironment(parent) )
+    if( !isEnvironment(parent) &&
+	!isEnvironment((parent = simple_as_environment(parent))))
 	error(_("'parent' is not an environment"));
 
     SET_ENCLOS(env, parent);
 
-    return( env );
+    return( CAR(args) );
 }
 
 SEXP attribute_hidden do_envirName(SEXP call, SEXP op, SEXP args, SEXP rho)
@@ -379,7 +377,8 @@ SEXP attribute_hidden do_envirName(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP env = CAR(args), ans=mkString(""), res;
 
     checkArity(op, args);
-    if (TYPEOF(env) == ENVSXP) {
+    if (TYPEOF(env) == ENVSXP ||
+	TYPEOF((env = simple_as_environment(env))) == ENVSXP) {
 	if (env == R_GlobalEnv) ans = mkString("R_GlobalEnv");
 	else if (env == R_BaseEnv) ans = mkString("base");
 	else if (env == R_EmptyEnv) ans = mkString("R_EmptyEnv");
@@ -387,30 +386,48 @@ SEXP attribute_hidden do_envirName(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    ans = ScalarString(STRING_ELT(R_PackageEnvName(env), 0));
 	else if (R_IsNamespaceEnv(env))
 	    ans = ScalarString(STRING_ELT(R_NamespaceEnvSpec(env), 0));
-	else if (!isNull(res = getAttrib(env, install("name")))) ans = res;
+	else if (!isNull(res = getAttrib(env, R_NameSymbol))) ans = res;
     }
     return ans;
 }
 
 #ifdef Win32
 # include "rgui_UTF8.h"
+#endif
 static const char *trChar(SEXP x)
 {
-    char *p;
-    static char buf[106];
-    int n = strlen(CHAR(x));
-    /* Long strings will be rare, and few per cat() call so we
-       can afford to be profligate here: translateChar is */
-    if (n < 100) p = buf; else p = R_alloc(n+7, 1);
-    if (WinUTF8out && getCharCE(x) == CE_UTF8) {
-	strcpy(p, UTF8in); strcat(p, CHAR(x)); strcat(p, UTF8out);
-	return p;
-    } else
-	return translateChar(x);
-}
-#else
-# define trChar(x) translateChar(x)
+    size_t n = strlen(CHAR(x));
+    cetype_t ienc = getCharCE(x);
+
+    if (ienc == CE_BYTES) {
+	const char *p = CHAR(x), *q;
+	char *pp = R_alloc(4*n+1, 1), *qq = pp, buf[5];
+	for (q = p; *q; q++) {
+	    unsigned char k = (unsigned char) *q;
+	    if (k >= 0x20 && k < 0x80) {
+		*qq++ = *q;
+	    } else {
+		snprintf(buf, 5, "\\x%02x", k);
+		for(int j = 0; j < 4; j++) *qq++ = buf[j];
+	    }
+	}
+	*qq = '\0';
+	return pp;
+    } else {
+#ifdef Win32
+	static char buf[106];
+	char *p;
+	/* Long strings will be rare, and few per cat() call so we
+	   can afford to be profligate here: translateChar is */
+	if (n < 100) p = buf; else p = R_alloc(n+7, 1);
+	if (WinUTF8out && ienc == CE_UTF8) {
+	    strcpy(p, UTF8in); strcat(p, CHAR(x)); strcat(p, UTF8out);
+	    return p;
+	} else
 #endif
+	    return translateChar(x);
+    }
+}
 
 static void cat_newline(SEXP labels, int *width, int lablen, int ntot)
 {
@@ -457,7 +474,8 @@ static void cat_cleanup(void *data)
 
     con->fflush(con);
     if(changedcon) switch_stdout(-1, 0);
-    if(!wasopen) con->close(con);
+    /* previous line might have closed it */
+    if(!wasopen && con->isopen) con->close(con);
 #ifdef Win32
     WinUTF8out = FALSE;
 #endif
@@ -478,10 +496,7 @@ SEXP attribute_hidden do_cat(SEXP call, SEXP op, SEXP args, SEXP rho)
     checkArity(op, args);
 
     /* Use standard printing defaults */
-    PrintDefaults(rho);
-#ifdef Win32
-    WinCheckUTF8();
-#endif
+    PrintDefaults();
 
     objs = CAR(args);
     args = CDR(args);
@@ -489,6 +504,8 @@ SEXP attribute_hidden do_cat(SEXP call, SEXP op, SEXP args, SEXP rho)
     file = CAR(args);
     ifile = asInteger(file);
     con = getConnection(ifile);
+    if(!con->canwrite) /* if it is not open, we may not know yet */
+	error(_("cannot write to this connection"));
     args = CDR(args);
 
     sepr = CAR(args);
@@ -528,7 +545,11 @@ SEXP attribute_hidden do_cat(SEXP call, SEXP op, SEXP args, SEXP rho)
     ci.wasopen = con->isopen;
 
     ci.changedcon = switch_stdout(ifile, 0);
-    /* will open new connection if required */
+    /* will open new connection if required, and check for writeable */
+#ifdef Win32
+    /* do this after re-sinking output */
+    WinCheckUTF8();
+#endif
 
     ci.con = con;
 
@@ -547,11 +568,12 @@ SEXP attribute_hidden do_cat(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if (iobj != 0 && !isNull(s))
 	    cat_printsep(sepr, 0);
 	n = length(s);
+	/* 0-length objects are ignored */
 	if (n > 0) {
 	    if (labs != R_NilValue && (iobj == 0)
 		&& (asInteger(fill) > 0)) {
 		Rprintf("%s ", trChar(STRING_ELT(labs, nlines % lablen)));
-		/* FIXME -- Rstrlen allows for embedded nuls, double-width chars */
+		/* FIXME -- Rstrlen allows for double-width chars */
 		width += Rstrlen(STRING_ELT(labs, nlines % lablen), 0) + 1;
 		nlines++;
 	    }
@@ -654,6 +676,7 @@ SEXP attribute_hidden do_makelist(SEXP call, SEXP op, SEXP args, SEXP rho)
     return list;
 }
 
+/* This is a primitive SPECIALSXP */
 SEXP attribute_hidden do_expression(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP a, ans, nms;
@@ -694,11 +717,11 @@ SEXP attribute_hidden do_makevector(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP s;
     SEXPTYPE mode;
     checkArity(op, args);
+    if (length(CADR(args)) != 1) error(_("invalid '%s' argument"), "length");
     len = asVecSize(CADR(args));
     if (len < 0) error(_("invalid '%s' argument"), "length");
     s = coerceVector(CAR(args), STRSXP);
-    if (length(s) == 0)
-	error(_("vector: zero-length 'type' argument"));
+    if (length(s) != 1) error(_("invalid '%s' argument"), "mode");
     mode = str2type(CHAR(STRING_ELT(s, 0))); /* ASCII */
     if (mode == -1 && streql(CHAR(STRING_ELT(s, 0)), "double"))
 	mode = REALSXP;
@@ -717,7 +740,7 @@ SEXP attribute_hidden do_makevector(SEXP call, SEXP op, SEXP args, SEXP rho)
 	s = allocList(len);
 	break;
     default:
-	error(_("vector: cannot make a vector of mode \"%s\"."),
+	error(_("vector: cannot make a vector of mode '%s'."),
 	      translateChar(STRING_ELT(s, 0))); /* should be ASCII */
     }
     if (mode == INTSXP || mode == LGLSXP)
@@ -727,7 +750,7 @@ SEXP attribute_hidden do_makevector(SEXP call, SEXP op, SEXP args, SEXP rho)
     else if (mode == CPLXSXP)
 	memset(COMPLEX(s), 0, len*sizeof(Rcomplex));
     else if (mode == RAWSXP)
-	memset(RAW(s), 0, len);
+	memset(RAW(s), 0, (size_t) len);
     /* other cases: list/expression have "NULL", ok */
     return s;
 }
@@ -746,7 +769,7 @@ SEXP lengthgets(SEXP x, R_len_t len)
     lenx = length(x);
     if (lenx == len)
 	return (x);
-    rval = allocVector(TYPEOF(x), len);
+    PROTECT(rval = allocVector(TYPEOF(x), len));
     PROTECT(xnames = getAttrib(x, R_NamesSymbol));
     if (xnames != R_NilValue)
 	names = allocVector(STRSXP, len);
@@ -825,7 +848,7 @@ SEXP lengthgets(SEXP x, R_len_t len)
     }
     if (isVector(x) && xnames != R_NilValue)
 	setAttrib(rval, R_NamesSymbol, names);
-    UNPROTECT(1);
+    UNPROTECT(2);
     return rval;
 }
 
@@ -836,9 +859,11 @@ SEXP attribute_hidden do_lengthgets(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP x, ans;
 
     checkArity(op, args);
+    check1arg(args, call, "x");
+
     x = CAR(args);
-    if( isObject(x) && DispatchOrEval(call, op, "length<-", args,
-				      rho, &ans, 0, 1))
+    if(isObject(x) && DispatchOrEval(call, op, "length<-", args,
+				     rho, &ans, 0, 1))
 	return(ans);
     if (!isVector(x) && !isVectorizable(x))
        error(_("invalid argument"));
@@ -851,68 +876,140 @@ SEXP attribute_hidden do_lengthgets(SEXP call, SEXP op, SEXP args, SEXP rho)
     return lengthgets(x, len);
 }
 
-
-
-/* For switch, evaluate the first arg, if it is a character then try */
-/* to match the name with the remaining args, and evaluate the match, */
-/* if there is no match then evaluate the first unnamed arg.  If the */
-/* value of the first arg is not a character string then coerce it to */
-/* an integer k and choose the kth argument from those that remain */
-/* provided 0 < k < (nargs-1). For character matching, if the value */
-/* is missing then take the next non-missing arg as the value. Then */
-/* things like switch(as.character(answer), yes=, YES=1, no=, NO=2, 3) */
-/* will work. */
-
-static SEXP switchList(SEXP el, SEXP rho)
+/* Expand dots in args, but do not evaluate */
+static SEXP expandDots(SEXP el, SEXP rho)
 {
-    SEXP h;
-    if (CAR(el) == R_DotsSymbol) {
-	h = findVar(CAR(el), rho);
-	if (h == R_NilValue)
-	    return R_NilValue;
-	if (TYPEOF(h) != DOTSXP) {
-	    if (h == R_MissingArg)
-		return R_MissingArg;
-	    error(_("... used in an incorrect context"));
+    SEXP ans, tail;
+
+    PROTECT(el); /* in do_switch, this is already protected */
+    PROTECT(ans = tail = CONS(R_NilValue, R_NilValue));
+
+    while (el != R_NilValue) {
+	if (CAR(el) == R_DotsSymbol) {
+	    SEXP h = findVar(CAR(el), rho);
+	    if (TYPEOF(h) == DOTSXP || h == R_NilValue) {
+		while (h != R_NilValue) {
+		    SETCDR(tail, CONS(CAR(h), R_NilValue));
+		    tail = CDR(tail);
+		    if(TAG(h) != R_NilValue) SET_TAG(tail, TAG(h));
+		    h = CDR(h);
+		}
+	    } else if (h != R_MissingArg)
+		error(_("'...' used in an incorrect context"));
+	} else {
+	    SETCDR(tail, CONS(CAR(el), R_NilValue));
+	    tail = CDR(tail);
+	    if(TAG(el) != R_NilValue) SET_TAG(tail, TAG(el));
 	}
-	return h;
-    }
-    else {
-	error(_("invalid parameter in 'switch()'"));
-	return R_NilValue;/* for -Wall */
+	el = CDR(el);
     }
+    UNPROTECT(2);
+    return CDR(ans);
 }
 
+/* This function is used in do_switch to record the default value and
+   to detect multiple defaults, which are not allowed as of 2.13.x */
+   
+static SEXP setDflt(SEXP arg, SEXP dflt) 
+{
+    if (dflt) {
+    	SEXP dflt1, dflt2;
+    	PROTECT(dflt1 = deparse1line(dflt, TRUE));
+    	PROTECT(dflt2 = deparse1line(CAR(arg), TRUE));
+    	error(_("duplicate switch defaults: '%s' and '%s'"), CHAR(STRING_ELT(dflt1, 0)),
+    	                                                   CHAR(STRING_ELT(dflt2, 0)));
+    	UNPROTECT(2); /* won't get here, but just for good form */
+    }
+    return(CAR(arg));
+}
+
+/* For switch, evaluate the first arg, if it is a character then try
+ to match the name with the remaining args, and evaluate the match. If
+ the value is missing then take the next non-missing arg as the value.
+ Then things like switch(as.character(answer), yes=, YES=1, no=, NO=2,
+ 3) will work.  But if there is no 'next', return NULL. One arg beyond
+ the first is allowed to be unnamed; it becomes the default value if
+ there is no match.
+ 
+ If the value of the first arg is not a character string
+ then coerce it to an integer k and choose the kth argument from those
+ that remain provided 1 < k < nargs.  
+
+ Changed in 2.11.0 to be primitive, so the wrapper does not partially
+ match to EXPR, and to return NULL invisibly if it is an error
+ condition.
+
+ This is a SPECIALSXP, so arguments need to be evaluated as needed.
+  And (see names.c) X=2, so it defaults to a visible value.
+*/
+
+
 SEXP attribute_hidden do_switch(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    int argval;
-    SEXP x, y, w;
-    x = eval(CAR(args), rho);
+    int argval, nargs = length(args);
+    SEXP x, y, z, w, ans, dflt = NULL;
+
+    if (nargs < 1) errorcall(call, _("'EXPR' is missing"));
+    check1arg(args, call, "EXPR");
+    PROTECT(x = eval(CAR(args), rho));
     if (!isVector(x) || length(x) != 1)
-	error(_("switch: EXPR must return a length 1 vector"));
-    PROTECT(w = switchList(CDR(args), rho));
-    if (isString(x)) {
-	for (y = w; y != R_NilValue; y = CDR(y))
-	    if (TAG(y) != R_NilValue && pmatch(STRING_ELT(x, 0), TAG(y), 1)) {
-		while (CAR(y) == R_MissingArg && y != R_NilValue)
-		    y = CDR(y);
-		UNPROTECT(1);
-		return (eval(CAR(y), rho));
+	errorcall(call, _("EXPR must be a length 1 vector"));
+    if (nargs > 1) {
+	/* There is a complication: if called from lapply
+	   there may be a ... argument */
+	PROTECT(w = expandDots(CDR(args), rho));
+	if (isString(x)) {
+	    for (y = w; y != R_NilValue; y = CDR(y)) {
+		if (TAG(y) != R_NilValue) {
+		    if (pmatch(STRING_ELT(x, 0), TAG(y), 1 /* exact */)) {
+			/* Find the next non-missing argument.
+			   (If there is none, return NULL.) */
+			while (CAR(y) == R_MissingArg) {
+			    y = CDR(y);
+			    if (y == R_NilValue) break;
+			    if (TAG(y) == R_NilValue) dflt = setDflt(y, dflt);
+			}
+			if (y == R_NilValue) {
+			    R_Visible = FALSE;
+			    UNPROTECT(2);
+			    return R_NilValue;
+			}
+			/* Check for multiple defaults following y.  This loop
+			   is not necessary to determine the value of the
+			   switch(), but it should be fast and will detect
+			   typos. */
+			for (z = CDR(y); z != R_NilValue; z = CDR(z)) 
+			    if (TAG(z) == R_NilValue) dflt = setDflt(z, dflt);
+			    
+			ans =  eval(CAR(y), rho);
+			UNPROTECT(2);
+			return ans;
+		    }
+		} else
+		    dflt = setDflt(y, dflt);
 	    }
-	for (y = w; y != R_NilValue; y = CDR(y))
-	    if (TAG(y) == R_NilValue) {
-		UNPROTECT(1);
-		return (eval(CAR(y), rho));
+ 	    if (dflt) {
+		ans =  eval(dflt, rho);
+		UNPROTECT(2);
+		return ans;
 	    }
-	UNPROTECT(1);
-	return R_NilValue;
+	    /* fall through to error */
+	} else { /* Treat as numeric */
+	    argval = asInteger(x);
+	    if (argval != NA_INTEGER && argval >= 1 && argval <= length(w)) {
+		SEXP alt = CAR(nthcdr(w, argval - 1));
+		if (alt == R_MissingArg)
+		    error("empty alternative in numeric switch");
+		ans =  eval(alt, rho);
+		UNPROTECT(2);
+		return ans;
+	    }
+	    /* fall through to error */
+	}
+	UNPROTECT(1); /* w */
     }
-    argval = asInteger(x);
-    if (argval <= 0 || argval > (length(w))) {
-	UNPROTECT(1);
-	return R_NilValue;
-    }
-    x = eval(CAR(nthcdr(w, argval - 1)), rho);
-    UNPROTECT(1);
-    return x;
+    /* an error */
+    UNPROTECT(1); /* x */
+    R_Visible = FALSE;
+    return R_NilValue;
 }
