@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-9   The R Development Core Team.
+ *  Copyright (C) 2001-11   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,14 @@
 static SEXP allocMatrixNA(SEXPTYPE, int, int);
 static void transferVector(SEXP s, SEXP t);
 
+static void con_cleanup(void *data)
+{
+    Rconnection con = data;
+    if(con->isopen) con->close(con);
+}
+
+static Rboolean field_is_foldable_p(const char *, SEXP);
+
 SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     int nwhat, nret, nc, nr, m, k, lastm, need;
@@ -40,31 +48,42 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP file, what, what2, retval, retval2, dims, dimnames;
     Rconnection con = NULL;
     Rboolean wasopen, is_eblankline;
+    RCNTXT cntxt;
+
+    SEXP fold_excludes;
+    Rboolean field_fold = TRUE, has_fold_excludes;
+    const char *field_name;
+    int offset = 0; /* -Wall */
 
     checkArity(op, args);
 
     file = CAR(args);
     con = getConnection(asInteger(file));
-    if(!con->canread)
-	error(_("cannot read from this connection"));
     wasopen = con->isopen;
     if(!wasopen) {
 	if(!con->open(con)) error(_("cannot open the connection"));
-	if(!con->canread) { /* recheck */
-	    con->close(con);
-	    error(_("cannot read from this connection"));
-	}
+	/* Set up a context which will close the connection on error */
+	begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+		     R_NilValue, R_NilValue);
+	cntxt.cend = &con_cleanup;
+	cntxt.cenddata = con;
     }
+    if(!con->canread) error(_("cannot read from this connection"));
 
-    PROTECT(what = coerceVector(CADR(args), STRSXP)); /* argument fields */
+    args = CDR(args);
+    PROTECT(what = coerceVector(CAR(args), STRSXP)); /* argument fields */
     nwhat = LENGTH(what);
     dynwhat = (nwhat == 0);
 
+    args = CDR(args);
+    PROTECT(fold_excludes = coerceVector(CAR(args), STRSXP));
+    has_fold_excludes = (LENGTH(fold_excludes) > 0);
+
     buf = (char *) malloc(buflen);
     if(!buf) error(_("could not allocate memory for 'read.dcf'"));
     nret = 20;
     /* it is easier if we first have a record per column */
-    PROTECT (retval = allocMatrixNA(STRSXP, LENGTH(what), nret));
+    PROTECT(retval = allocMatrixNA(STRSXP, LENGTH(what), nret));
 
     tre_regcomp(&blankline, "^[[:blank:]]*$", REG_NOSUB & REG_EXTENDED);
     tre_regcomp(&trailblank, "[[:blank:]]+$", REG_EXTENDED);
@@ -92,53 +111,77 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 		blank_skip = TRUE;
 		lastm = -1;
 		field_skip = FALSE;
+		field_fold = TRUE;
 	    }
 	} else {
 	    blank_skip = FALSE;
-	    /* Remove trailing whitespace. */
-	    if(tre_regexecb(&trailblank, line, 1, regmatch, 0) == 0)
-		line[regmatch[0].rm_so] = '\0';
 	    if(tre_regexecb(&contline, line, 1, regmatch, 0) == 0) {
 		/* A continuation line: wrong if at the beginning of a
 		   record. */
-		if(lastm == -1 && !field_skip) {
+		if((lastm == -1) && !field_skip) {
 		    line[20] = '\0';
 		    error(_("Found continuation line starting '%s ...' at begin of record."),
 			  line);
-		    continue;
 		}
 		if(lastm >= 0) {
 		    need = strlen(CHAR(STRING_ELT(retval,
-						  lastm + nwhat*k))) + 2;
+						  lastm + nwhat * k))) + 2;
 		    if(tre_regexecb(&eblankline, line, 0, NULL, 0) == 0) {
 			is_eblankline = TRUE;
 		    } else {
 			is_eblankline = FALSE;
-			need += strlen(line+regmatch[0].rm_eo);
+			if(field_fold) {
+			    offset = regmatch[0].rm_eo;
+			    /* Also remove trailing whitespace. */
+			    if((tre_regexecb(&trailblank, line, 1,
+					     regmatch, 0) == 0))
+				line[regmatch[0].rm_so] = '\0';
+			} else {
+			    offset = 0;
+			}
+			need += strlen(line + offset);
 		    }
 		    if(buflen < need) {
-			buf = (char *) realloc(buf, need);
-			if(!buf)
+			char *tmp = (char *) realloc(buf, need);
+			if(!tmp) {
+			    free(buf);
 			    error(_("could not allocate memory for 'read.dcf'"));
+			} else buf = tmp;
 			buflen = need;
 		    }
-		    strcpy(buf,CHAR(STRING_ELT(retval, lastm + nwhat*k)));
+		    strcpy(buf,CHAR(STRING_ELT(retval, lastm + nwhat * k)));
 		    strcat(buf, "\n");
-		    if(!is_eblankline) strcat(buf, line+regmatch[0].rm_eo);
-		    SET_STRING_ELT(retval, lastm + nwhat*k, mkChar(buf));
+		    if(!is_eblankline) strcat(buf, line + offset);
+		    SET_STRING_ELT(retval, lastm + nwhat * k, mkChar(buf));
 		}
 	    } else {
-		if(tre_regexecb(&regline, line, 1, regmatch, 0) == 0){
+		if(tre_regexecb(&regline, line, 1, regmatch, 0) == 0) {
 		    for(m = 0; m < nwhat; m++){
 			whatlen = strlen(CHAR(STRING_ELT(what, m)));
 			if(strlen(line) > whatlen &&
 			   line[whatlen] == ':' &&
 			   strncmp(CHAR(STRING_ELT(what, m)),
 				   line, whatlen) == 0) {
-			    SET_STRING_ELT(retval, m+nwhat*k,
-					   mkChar(line + regmatch[0].rm_eo));
+			    /* An already known field we are recording. */
 			    lastm = m;
 			    field_skip = FALSE;
+			    field_name = CHAR(STRING_ELT(what, lastm));
+			    if(has_fold_excludes) {
+				field_fold =
+				    field_is_foldable_p(field_name,
+							fold_excludes);
+			    }
+			    if(field_fold) {
+				offset = regmatch[0].rm_eo;
+				/* Also remove trailing whitespace. */
+				if((tre_regexecb(&trailblank, line, 1,
+						 regmatch, 0) == 0))
+				    line[regmatch[0].rm_so] = '\0';
+			    } else {
+				offset = 0;
+			    }
+			    SET_STRING_ELT(retval, m + nwhat * k,
+					   mkChar(line + offset));
 			    break;
 			} else {
 			    /* This is a field, but not one prespecified */
@@ -148,7 +191,7 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 		    }
 		    if(dynwhat && (lastm == -1)) {
 			/* A previously unseen field and we are
-			   recording all fields */
+			 * recording all fields */
 			field_skip = FALSE;
 			PROTECT(what2 = allocVector(STRSXP, nwhat+1));
 			PROTECT(retval2 = allocMatrixNA(STRSXP,
@@ -168,11 +211,20 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 			UNPROTECT_PTR(what);
 			retval = retval2;
 			what = what2;
+			/* FIXME:
+			   Why are we doing this?
+			   We need to copy the matched beginning of the
+			   line to buf, so shouldn't we need
+			     regmatch[0].rm_eo
+			   bytes?
+			*/
 			need = strlen(line+regmatch[0].rm_eo);
 			if(buflen < need){
-			    buf = (char *) realloc(buf, need);
-			    if(!buf)
+			    char *tmp = (char *) realloc(buf, need);
+			    if(!tmp) {
+				free(buf);
 				error(_("could not allocate memory for 'read.dcf'"));
+			    } else buf = tmp;
 			    buflen = need;
 			}
 			strncpy(buf, line, Rf_strchr(line, ':') - line);
@@ -181,8 +233,21 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 			nwhat++;
 			/* lastm uses C indexing, hence nwhat - 1 */
 			lastm = nwhat - 1;
-			SET_STRING_ELT(retval, lastm + nwhat*k,
-				       mkChar(line + regmatch[0].rm_eo));
+			field_name = CHAR(STRING_ELT(what, lastm));
+			if(has_fold_excludes) {
+			    field_fold =
+				field_is_foldable_p(field_name,
+						    fold_excludes);
+			}
+			offset = regmatch[0].rm_eo;
+			if(field_fold) {
+			    /* Also remove trailing whitespace. */
+			    if((tre_regexecb(&trailblank, line, 1,
+					     regmatch, 0) == 0))
+				line[regmatch[0].rm_so] = '\0';
+			}
+			SET_STRING_ELT(retval, lastm + nwhat * k,
+				       mkChar(line + offset));
 		    }
 		} else {
 		    /* Must be a regular line with no tag ... */
@@ -192,7 +257,7 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	}
     }
-    if(!wasopen) con->close(con);
+    if(!wasopen) {endcontext(&cntxt); con->close(con);}
     free(buf);
     tre_regfree(&blankline);
     tre_regfree(&contline);
@@ -213,7 +278,7 @@ SEXP attribute_hidden do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
     SET_VECTOR_ELT(dimnames, 1, what);
     setAttrib(retval2, R_DimSymbol, dims);
     setAttrib(retval2, R_DimNamesSymbol, dimnames);
-    UNPROTECT(5);
+    UNPROTECT(6);
     return(retval2);
 }
 
@@ -235,10 +300,16 @@ static SEXP allocMatrixNA(SEXPTYPE mode, int nrow, int ncol)
 
 static void transferVector(SEXP s, SEXP t)
 {
-    int i, ns, nt;
-
-    nt = LENGTH(t);
-    ns = LENGTH(s);
-    for (i = 0; i < nt; i++)
+    for (int i = 0; i < LENGTH(t); i++)
 	SET_STRING_ELT(s, i, STRING_ELT(t, i));
 }
+
+static Rboolean field_is_foldable_p(const char *field, SEXP excludes)
+{
+    int i, n = LENGTH(excludes);
+    for(i = 0; i < n; i++) {
+	if(strcmp(field, CHAR(STRING_ELT(excludes, i))) == 0)
+	    return FALSE;
+    }
+    return TRUE;
+}
