@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1999-2008  Guido Masarotto and the R Development Core Team
+ *  Copyright (C) 1999-2010  Guido Masarotto and the R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,6 +48,8 @@
 #define GETBLUE(col)   (((col) >> BSHIFT) & 0xFF)
 #define GETALPHA(col)   (((col) >> 24) & 0xFF)
 
+#include "bitmap.h"
+
 #ifdef HAVE_PNG
 #include "png.h"
 #include <setjmp.h>
@@ -70,7 +72,11 @@
 static void my_png_error(png_structp png_ptr, png_const_charp msg)
 {
     R_ShowMessage((char *) msg);
+#if PNG_LIBPNG_VER < 10400
     longjmp(png_ptr->jmpbuf,1);
+#else
+    longjmp(png_jmpbuf(png_ptr),1);
+#endif
 }
 
 static void my_png_warning(png_structp png_ptr, png_const_charp msg)
@@ -86,16 +92,22 @@ int R_SaveAsPng(void  *d, int width, int height,
     png_infop info_ptr;
     unsigned int  col, palette[256];
     png_color pngpalette[256];
-    png_bytep pscanline, scanline = (png_bytep) calloc(4*width,sizeof(png_byte));
+    png_bytep pscanline;
+    png_bytep scanline = (png_bytep) calloc((size_t)(4*width),sizeof(png_byte));
     png_byte trans[256];
     png_color_16 trans_values[1];
     int i, j, r, ncols, mid, high, low, withpalette, have_alpha;
-    DECLARESHIFTS;
+    volatile DECLARESHIFTS;
 
     /* Have we enough memory?*/
     if (scanline == NULL)
 	return 0;
 
+    if (fp == NULL) {
+	free(scanline);
+	return 0;
+    }
+
     /* Create and initialize the png_struct with the desired error handler
      * functions.  If you want to use the default stderr and longjump method,
      * you can supply NULL for the last three parameters.  We also check that
@@ -119,7 +131,12 @@ int R_SaveAsPng(void  *d, int width, int height,
     /* Set error handling.  REQUIRED if you aren't supplying your own
      * error handling functions in the png_create_write_struct() call.
      */
-    if (setjmp(png_ptr->jmpbuf)) {
+#if PNG_LIBPNG_VER < 10400
+    if (setjmp(png_ptr->jmpbuf))
+#else
+    if (setjmp(png_jmpbuf(png_ptr)))
+#endif
+{
 	/* If we get here, we had a problem writing the file */
 	free(scanline);
 	png_destroy_write_struct(&png_ptr, &info_ptr);
@@ -277,7 +294,7 @@ int R_SaveAsPng(void  *d, int width, int height,
 #else
 int R_SaveAsPng(void  *d, int width, int height,
 		unsigned int (*gp)(void *, int, int),
-		int bgr, FILE *fp, unsigned int transparent)
+		int bgr, FILE *fp, unsigned int transparent, int res)
 {
     warning("No png support in this version of R");
     return 0;
@@ -343,12 +360,17 @@ int R_SaveAsJpeg(void  *d, int width, int height,
     JSAMPLE *pscanline, *scanline = (JSAMPLE *) calloc(3*width,sizeof(JSAMPLE));
     int i, j;
     unsigned int col;
-    DECLARESHIFTS;
+    volatile DECLARESHIFTS;
 
     /* Have we enough memory?*/
     if (scanline == NULL)
 	return 0;
 
+    if (outfile == NULL) {
+	free(scanline);
+	return 0;
+    }
+
     /* Step 1: allocate and initialize JPEG compression object */
 
     /*
@@ -567,6 +589,9 @@ int R_SaveAsBmp(void  *d, int width, int height,
     int lres;
     DECLARESHIFTS;
 
+    if (fp == NULL)
+	return 0;
+
     /* Have we less than 256 different colors? */
     ncols = mid = 0;
     withpalette = 1;
