@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2009 The R Development Core Team.
+ *  Copyright (C) 2009-11 The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -166,6 +166,12 @@ static struct sockaddr *build_sin(struct sockaddr_in *sa, const char *ip, int po
 #define CONTENT_TYPE      0x40 /* message has a specific content type set */
 #define CONTENT_FORM_UENC 0x80 /* message content type is application/x-www-form-urlencoded */
 
+struct buffer {
+    struct buffer *next, *prev;
+    int size, length;
+    char data[1];
+};
+
 /* --- connection/worker structure holding all data for an active connection --- */
 typedef struct httpd_conn {
     SOCKET sock;         /* client socket */
@@ -178,8 +184,10 @@ typedef struct httpd_conn {
     char line_buf[LINE_BUF_SIZE];  /* line buffer (used for request and headers) */
     char *url, *body;              /* URL and request body */
     char *content_type;            /* content type (if set) */
-    unsigned int line_pos, body_pos, content_length; /* positions in the buffers and desired content length */
+    unsigned int line_pos, body_pos; /* positions in the buffers */
+    long content_length;           /* desired content length */
     char part, method, attr;       /* request part, method and connection attributes */
+    struct buffer *headers;        /* buffer holding header lines */
 } httpd_conn_t;
 
 #define IS_HTTP_1_1(C) (((C)->attr & HTTP_1_0) == 0)
@@ -220,6 +228,45 @@ static void first_init()
     needs_init = 0;
 }
 
+/* free buffers starting from the tail(!!) */
+static void free_buffer(struct buffer *buf) {
+    if (!buf) return;
+    if (buf->prev) free_buffer(buf->prev);
+    free(buf);
+}
+
+/* allocate a new buffer */
+static struct buffer *alloc_buffer(int size, struct buffer *parent) {
+    struct buffer *buf = (struct buffer*) malloc(sizeof(struct buffer) + size);
+    if (!buf) return buf;
+    buf->next = 0;
+    buf->prev = parent;
+    if (parent) parent->next = buf;
+    buf->size = size;
+    buf->length = 0;
+    return buf;
+}
+
+/* convert doubly-linked buffers into one big raw vector */
+static SEXP collect_buffers(struct buffer *buf) {
+    SEXP res;
+    char *dst;
+    int len = 0;
+    if (!buf) return allocVector(RAWSXP, 0);
+    while (buf->prev) { /* count the total length and find the root */
+	len += buf->length;
+	buf = buf->prev;
+    }
+    res = allocVector(RAWSXP, len + buf->length);
+    dst = (char*) RAW(res);
+    while (buf) {
+	memcpy(dst, buf->data, buf->length);
+	dst += buf->length;
+	buf = buf->next;
+    }
+    return res;
+}
+
 static void finalize_worker(httpd_conn_t *c)
 {
     DBG(printf("finalizing worker %p\n", (void*) c));
@@ -243,7 +290,10 @@ static void finalize_worker(httpd_conn_t *c)
 	free(c->content_type);
 	c->content_type = NULL;
     }
-    
+    if (c->headers) {
+	free_buffer(c->headers);
+	c->headers = NULL;
+    }
     if (c->sock != INVALID_SOCKET) {
 	closesocket(c->sock);
 	c->sock = INVALID_SOCKET;
@@ -415,6 +465,8 @@ static SEXP parse_query(char *query)
     return res;
 }
 
+static SEXP R_ContentTypeName, R_HandlersName;
+
 /* create an object representing the request body. It is NULL if the body is empty (or zero length).
  * In the case of a URL encoded form it will have the same shape as the query string (named string vector).
  * In all other cases it will be a raw vector with a "content-type" attribute (if specified in the headers) */
@@ -428,8 +480,10 @@ static SEXP parse_request_body(httpd_conn_t *c) {
 	SEXP res = PROTECT(Rf_allocVector(RAWSXP, c->content_length));
 	if (c->content_length)
 	    memcpy(RAW(res), c->body, c->content_length);
-	if (c->content_type) /* attach the content type so it can be interpreted */
-	    setAttrib(res, install("content-type"), mkString(c->content_type));
+	if (c->content_type) { /* attach the content type so it can be interpreted */
+	    if (!R_ContentTypeName) R_ContentTypeName = install("content-type");
+	    setAttrib(res, R_ContentTypeName, mkString(c->content_type));
+	}
 	UNPROTECT(1);
 	return res;
     }
@@ -475,8 +529,10 @@ static SEXP handler_for_path(const char *path) {
 	    fn[c - e] = 0;
 	    DBG(Rprintf("handler_for_path('%s'): looking up custom handler '%s'\n", path, fn));
 	    /* we cache custom_handlers_env so in case it has not been loaded yet, fetch it */
-	    if (!custom_handlers_env)
-		custom_handlers_env = eval(install(".httpd.handlers.env"), R_FindNamespace(mkString("tools")));
+	    if (!custom_handlers_env) {
+		if (!R_HandlersName) R_HandlersName = install(".httpd.handlers.env");
+		custom_handlers_env = eval(R_HandlersName, R_FindNamespace(mkString("tools")));
+	    }
 	    /* we only proceed if .httpd.handlers.env really exists */
 	    if (TYPEOF(custom_handlers_env) == ENVSXP) {
 		SEXP cl = findVarInFrame3(custom_handlers_env, install(fn), TRUE);
@@ -507,14 +563,17 @@ static void process_request(httpd_conn_t *c)
     uri_decode(c->url); /* decode the path part */
     {   /* construct "try(httpd(url, query, body), silent=TRUE)" */
 	SEXP sTrue = PROTECT(ScalarLogical(TRUE));
-	SEXP y, x = PROTECT(lang3(
-				  install("try"),
-				  LCONS(handler_for_path(c->url),
-					list3(mkString(c->url), query ? parse_query(query) : R_NilValue, parse_request_body(c))),
+	SEXP sBody = PROTECT(parse_request_body(c));
+	SEXP sQuery = PROTECT(query ? parse_query(query) : R_NilValue);
+	SEXP sReqHeaders = PROTECT(c->headers ? collect_buffers(c->headers) : R_NilValue);
+	SEXP sArgs = PROTECT(list4(mkString(c->url), sQuery, sBody, sReqHeaders));
+	SEXP sTry = install("try");
+	SEXP y, x = PROTECT(lang3(sTry,
+				  LCONS(handler_for_path(c->url), sArgs),
 				  sTrue));
 	SET_TAG(CDR(CDR(x)), install("silent"));
 	DBG(Rprintf("eval(try(httpd('%s'),silent=TRUE))\n", c->url));
-
+	
 	/* evaluate the above in the tools namespace */
 	x = PROTECT(eval(x, R_FindNamespace(mkString("tools"))));
 
@@ -546,7 +605,7 @@ static void process_request(httpd_conn_t *c)
 	    if (c->method != METHOD_HEAD)
 		send_response(c->sock, s, strlen(s));
 	    c->attr |= CONNECTION_CLOSE; /* force close */
-	    UNPROTECT(3);
+	    UNPROTECT(7);
 	    return;
 	}
 
@@ -595,7 +654,7 @@ static void process_request(httpd_conn_t *c)
 		    long fsz = 0;
 		    if (!f) {
 			send_response(c->sock, "\r\nContent-length: 0\r\n\r\n", 23);
-			UNPROTECT(3);
+			UNPROTECT(7);
 			fin_request(c);
 			return;
 		    }
@@ -611,7 +670,7 @@ static void process_request(httpd_conn_t *c)
 				int rd = (fsz > 32768) ? 32768 : fsz;
 				if (fread(fbuf, 1, rd, f) != rd) {
 				    free(fbuf);
-				    UNPROTECT(3);
+				    UNPROTECT(7);
 				    c->attr |= CONNECTION_CLOSE;
 				    return;
 				}
@@ -620,13 +679,13 @@ static void process_request(httpd_conn_t *c)
 			    }
 			    free(fbuf);
 			} else { /* allocation error - get out */
-			    UNPROTECT(3);
+			    UNPROTECT(7);
 			    c->attr |= CONNECTION_CLOSE;
 			    return;
 			}
 		    }
 		    fclose(f);
-		    UNPROTECT(3);
+		    UNPROTECT(7);
 		    fin_request(c);
 		    return;
 		}
@@ -634,7 +693,7 @@ static void process_request(httpd_conn_t *c)
 		send_response(c->sock, buf, strlen(buf));
 		if (c->method != METHOD_HEAD)
 		    send_response(c->sock, cs, strlen(cs));
-		UNPROTECT(3);
+		UNPROTECT(7);
 		fin_request(c);
 		return;
 	    }
@@ -660,12 +719,12 @@ static void process_request(httpd_conn_t *c)
 		send_response(c->sock, buf, strlen(buf));
 		if (c->method != METHOD_HEAD)
 		    send_response(c->sock, (char*) cs, LENGTH(y));
-		UNPROTECT(3);
+		UNPROTECT(7);
 		fin_request(c);
 		return;
 	    }
 	}
-	UNPROTECT(3);
+	UNPROTECT(7);
     }
     send_http_response(c, " 500 Invalid response from R\r\nConnection: close\r\nContent-type: text/plain\r\n\r\nServer error: invalid response from R\r\n");
     c->attr |= CONNECTION_CLOSE; /* force close */
@@ -723,8 +782,9 @@ static void worker_input_handler(void *data) {
 		    return;
 		}
 		if (c->attr & CONTENT_LENGTH && c->content_length) {
-		    c->body = (char*) malloc(c->content_length + 1); /* allocate an extra termination byte */
-		    if (!c->body) { /* uh oh - out of memory - refuse */
+		    if (c->content_length < 0 ||  /* we are parsing signed so negative numbers are bad */
+			c->content_length > 2147483640 || /* R will currently have issues with body around 2Gb or more, so better to not go there */
+			!(c->body = (char*) malloc(c->content_length + 1 /* allocate an extra termination byte */ ))) {
 			send_http_response(c, " 413 Request Entity Too Large (request body too big)\r\nConnection: close\r\n\r\n");
 			remove_worker(c);
 			return;
@@ -752,6 +812,7 @@ static void worker_input_handler(void *data) {
 		    if (c->url) { free(c->url); c->url = NULL; }
 		    if (c->body) { free(c->body); c->body = NULL; }
 		    if (c->content_type) { free(c->content_type); c->content_type = NULL; }
+		    if (c->headers) { free_buffer(c->headers); c->headers = NULL; }
 		    c->body_pos = 0;
 		    c->method = 0;
 		    c->part = PART_REQUEST;
@@ -813,6 +874,27 @@ static void worker_input_handler(void *data) {
 		    } else if (c->part == PART_HEADER) {
 			/* --- process headers --- */
 			char *k = bol;
+			if (!c->headers)
+			    c->headers = alloc_buffer(1024, NULL);
+			if (c->headers) { /* record the header line in the buffer */
+			    int l = strlen(bol);
+			    if (l) { /* this should be really always true */
+				if (c->headers->length + l + 1 > c->headers->size) { /* not enough space? */
+				    int fits = c->headers->size - c->headers->length;
+				    if (fits) memcpy(c->headers->data + c->headers->length, bol, fits);
+				    if (alloc_buffer(2048, c->headers)) {
+					c->headers = c->headers->next;
+					memcpy(c->headers->data, bol + fits, l - fits);
+					c->headers->length = l - fits;
+					c->headers->data[c->headers->length++] = '\n';
+				    }
+				} else {
+				    memcpy(c->headers->data + c->headers->length, bol, l);
+				    c->headers->length += l;	
+				    c->headers->data[c->headers->length++] = '\n';
+				}
+			    }
+			}
 			while (*k && *k != ':') {
 			    if (*k >= 'A' && *k <= 'Z')
 				*k |= 0x20;
@@ -824,7 +906,7 @@ static void worker_input_handler(void *data) {
 			    DBG(printf("header '%s' => '%s'\n", bol, k));
 			    if (!strcmp(bol, "content-length")) {
 				c->attr |= CONTENT_LENGTH;
-				c->content_length = atoi(k);
+				c->content_length = atol(k);
 			    }
 			    if (!strcmp(bol, "content-type")) {
 				char *l = k;
@@ -841,7 +923,7 @@ static void worker_input_handler(void *data) {
 				char *l = k;
 				while (*l) { if (*l >= 'A' && *l <= 'Z') *l |= 0x20; l++; }
 				if (!strncmp(k, "close", 5))
-				    c->attr = CONNECTION_CLOSE;
+				    c->attr |= CONNECTION_CLOSE;
 			    }
 			}
 		    }
@@ -856,9 +938,9 @@ static void worker_input_handler(void *data) {
     }
     if (c->part == PART_BODY && c->body) { /* BODY  - this branch always returns */
 	if (c->body_pos < c->content_length) { /* need to receive more ? */
-	    DBG(printf("BODY: body_pos=%d, content_length=%d\n", c->body_pos, c->content_length));
+	    DBG(printf("BODY: body_pos=%d, content_length=%ld\n", c->body_pos, c->content_length));
 	    n = recv(c->sock, c->body + c->body_pos, c->content_length - c->body_pos, 0);
-	    DBG(printf("      [recv n=%d - had %u of %u]\n", n, c->body_pos, c->content_length));
+	    DBG(printf("      [recv n=%d - had %u of %lu]\n", n, c->body_pos, c->content_length));
 	    c->line_pos = 0;
 	    if (n < 0) { /* error, scrap this worker */
 		remove_worker(c);
@@ -881,6 +963,7 @@ static void worker_input_handler(void *data) {
 	    if (c->url) { free(c->url); c->url = NULL; }
 	    if (c->body) { free(c->body); c->body = NULL; }
 	    if (c->content_type) { free(c->content_type); c->content_type = NULL; }
+	    if (c->headers) { free_buffer(c->headers); c->headers = NULL; }
 	    c->line_pos = 0; c->body_pos = 0;
 	    c->method = 0;
 	    c->part = PART_REQUEST;
@@ -917,6 +1000,7 @@ static void worker_input_handler(void *data) {
 		if (c->url) { free(c->url); c->url = NULL; }
 		if (c->body) { free(c->body); c->body = NULL; }
 		if (c->content_type) { free(c->content_type); c->content_type = NULL; }
+		if (c->headers) { free_buffer(c->headers); c->headers = NULL; }
 		c->body_pos = 0;
 		c->method = 0;
 		c->part = PART_REQUEST;
