@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *  DESCRIPTION
  *
@@ -39,8 +39,7 @@ double pweibull(double x, double shape, double scale, int lower_tail, int log_p)
     if (lower_tail)
 	return (log_p
 		/* log(1 - exp(x))  for x < 0 : */
-		? (x > -M_LN2 ? log(-expm1(x)) : log1p(-exp(x)))
-		: -expm1(x));
+		? R_Log1_Exp(x) : -expm1(x));
     /* else:  !lower_tail */
     return R_D_exp(x);
 }
