@@ -27,7 +27,7 @@ known <- c(32:126, 160:255)
 for(i in known) {
     x <- i %% 16
     y <- i %/% 16
-    points(x, y, pch=i)
+    points(x, y, pch=-i)
 }
 
 par(pty="m")
@@ -98,3 +98,50 @@ legend(1,10, c("A","bcd"), lwd = 2:3, pch= 21:22, pt.bg="skyblue",
 x <- 2^seq(1,1001, length=20)
 plot(x, x^0.9, type="l", log="xy")
 ## gave error 'Infinite axis extents [GEPretty(1.87013e-12,inf,5)]' for R 2.0.1
+
+plot(as.Date("2001/1/1") + 12*(1:9), 1:9)
+## used bad 'xlab/ylab' in some versions of R 2.2.0(unstable)
+
+## dotchart() restoring par()
+Opar <- par(no.readonly=TRUE) ; dotchart(1:4, cex= 0.7)
+Npar <- par(no.readonly=TRUE)
+ii <- c(37, 50:51, 58:59, 63)
+stopifnot(identical(names(Opar)[ii],
+                    c("mai","pin","plt","usr","xaxp","yaxp")),
+          identical(Opar[-ii], Npar[-ii]))
+## did not correctly restore par("mar") up to (incl) R 2.4.0
+
+## plot.function()     [n=11, ... : since we store and diff PS file !]
+plot(cos,       xlim=c(-5,5), n=11, axes=FALSE); abline(v=0)
+## did *not* plot for negative x up to R 2.5.1
+plot(sin, -2,3, xlim=c(-5,5), n=11, axes=FALSE, xlab="")# plot from -2
+axis(1, at=c(-2,3), tcl=-1); axis(1, at=c(-5,5))
+## (from,to) & xlim  should work simultaneously
+
+plot(cos, -7,7, n=11, axes=FALSE)
+## gave wrong ylab in R 2.6.0
+plot(cos, -7,7, ylab = "Cosine  cos(x)", n=11, axes=FALSE)
+## partial matching of 'ylab'; mapping  [0,1] (not [-7.7]):
+## margins chosen to avoid rouding error showing to 2dp.
+op <- par(mar=c(5,4.123,4,2)+0.1)
+plot(gamma, yla = expression(Gamma(x)), n=11, yaxt="n")
+par(op)
+
+## plot.ts(x, y) could get the labels wrong in R <= 2.6.0:
+x <- ts(1:5);x1 <- lag(x, 2); plot(x1, x, axes=FALSE)
+
+# adding a curve in log scale :
+curve(5*exp(-x), 0.1, 100, n = 3, log="x", ylab="", axes=FALSE)
+curve(5*exp(-x), add=TRUE, n = 3, col=2,lwd=3)
+## should fully overplot; wrong default xlim in 2.6.1
+## (and *slightly* wrong up to 2.6.0)
+
+## Axis() calls via plot()  {[xy]axt to keep *.ps small}
+x <- as.Date("2008-04-22 09:45") + (i <- c(0,4))
+plot(x,    xaxt="n")# not ok in 2.6.2, nor 2.7.0
+plot(x, i, yaxt="n")# ok in 2.6.2  and 2.7.0
+plot(i, x, xaxt="n")# ok in 2.6.2 and not in 2.7.0
+
+## table methods should be bypassed:
+dotchart(table(infert$education))
+## failed in 2.12.[12]
