@@ -1,7 +1,7 @@
-postscript("reg-tests-1b.ps", encoding = "ISOLatin1.enc")
+pdf("reg-tests-1b.pdf", encoding = "ISOLatin1.enc")
 
 ## force standard handling for data frames
-options(stringsAsFactors=TRUE)
+options(stringsAsFactors = TRUE)
 ## .Machine
 (Meps <- .Machine$double.eps)# and use it in this file
 
@@ -15,10 +15,13 @@ x <- structure(list(2), class="foo")
 str(x)
 ## gave infinite recursion < 2.6.0
 
+
 curve(sin, -2*pi, 3*pi); pu1 <- par("usr")[1:2]
-curve(cos); stopifnot(all.equal(par("usr")[1:2], pu1))
+curve(cos, add = NA) # add = NA new in 2.14.0
+stopifnot(all.equal(par("usr")[1:2], pu1))
 ## failed in R <= 2.6.0
 
+
 ## tests of side-effects with CHARSXP caching
 x <- y <- "abc"
 Encoding(x) <- "UTF-8"
@@ -184,7 +187,7 @@ tapply(character(0), factor(letters)[FALSE], length)
 
 
 ## zero-length patterns in gregexpr
-expect <- structure(1:3, match.length=rep(0L, 3))
+expect <- structure(1:3, match.length=rep(0L, 3), useBytes = TRUE)
 stopifnot(identical(expect, gregexpr("", "abc")[[1]]))
 stopifnot(identical(expect, gregexpr("", "abc", fixed=TRUE)[[1]]))
 stopifnot(identical(expect, gregexpr("", "abc", perl=TRUE)[[1]]))
@@ -290,11 +293,11 @@ dimnames(n)[[1]] <- c("a", "b")
 ## glob2rx(pattern, .) with "(", "[" or "{" in pattern :
 nm <- "my(ugly[file{name"
 stopifnot(identical(regexpr(glob2rx("*[*"), nm),
-		    structure(1L, match.length = 8L)),
+		    structure(1L, match.length = 8L, useBytes = TRUE)),
 	  identical(regexpr(glob2rx("*{n*"), nm),
-		    structure(1L, match.length = 14L)),
+		    structure(1L, match.length = 14L, useBytes = TRUE)),
 	  identical(regexpr(glob2rx("*y(*{*"), nm),
-		    structure(1L, match.length = 13L))
+		    structure(1L, match.length = 13L, useBytes = TRUE))
 	  )
 ## gave 'Invalid regular expression' in R <= 2.7.0
 
@@ -342,6 +345,7 @@ stopifnot(inherits(t1, "try-error"),
 	  identical(dd$x, dd[,"xx"]))
 ## From 2.5.0 to 2.7.1, the non-match indexing gave NULL instead of error
 
+
 ## data.frame[ (<NA>), ] when row.names had  "NA"
 x <- data.frame(x=1:3, y=2:4, row.names=c("a","b","NA"))
 y  <- x [c(2:3, NA),]
@@ -349,9 +353,11 @@ y.ok <- data.frame(x=c(2:3,NA), y=c(3:4,NA), row.names=c("b", "NA", "NA.1"))
 stopifnot(identical(y, y.ok))
 ## From 2.5.0 to 2.7.1,  y had row name "NA" twice
 
+
 stopifnot(shapiro.test(c(0,0,1))$p.value >= 0)
 ## was wrong up to 2.7.1, because of rounding errors (in single precision).
 
+
 stopifnot(rcond(cbind(1, c(3,3))) == 0)
 ## gave an error (because Lapack's LU detects exact singularity)
 
@@ -388,7 +394,7 @@ Data <- data.frame(x=c(1,1,1,1,1,2,2,3,3,3,3,3,3,4,4,4,5,5,5,5,6,6,6,6,6,6,
                    181,152,188,173,196,180,171,188,174,198, 172, 176,162,188,
                    182,182,141,191,190,159,170,163,197),
                    weight=c(1, rep(0.1, 51)))
-G.st <- c(k=0.005, g1=50,g2=550)
+G.st <- c(k=0.005, g1=50, g2=550)
 # model has length-1 (and 52) variables
 Ta <- min(Data$x)
 Tb <- max(Data$x)
@@ -398,9 +404,8 @@ nls(y~((g1)*exp((log(g2/g1))*(1-exp(-k*(x-Ta)))
                 /(1-exp(-k*(Tb-Ta))))), data=Data, start=G.st, trace=TRUE)
 
 #with weights
-nls(y~((g1)*exp((log(g2/g1))*(1-exp(-k*(x-Ta)))
-                /(1-exp(-k*(Tb-Ta))))), data=Data, start=G.st,
-    trace=TRUE, weights=weight)
+nls(y ~ ((g1)*exp((log(g2/g1))*(1-exp(-k*(x-Ta)))/(1-exp(-k*(Tb-Ta))))),
+    data = Data, start = G.st, trace = TRUE, weights = weight)
 ## failed for find weights in R <= 2.7.1
 
 
@@ -448,6 +453,7 @@ unlink("foo.txt")
 stopifnot(identical(z, c(1,2)))
 ## changed in 2.7.2 patched
 
+
 ## cov / cor / var etc with NAs :
 stopifnot(inherits(try(var(NULL)), "try-error"))## gave NA in 1.2.2
 v0 <- var(0[FALSE]) # gave "'x' is empty" in the past;  NA in 1.2.2
@@ -461,6 +467,7 @@ stopifnot(is.NA(v1), is.NA(v2), is.NA(sx),
 	  all.equal(0.5, var(x, na.rm=TRUE), tol=8*Meps)# should even be exact
 	  )
 
+
 ## write.dcf() indenting for ".<foo>" (PR#12816)
 zz <- textConnection("foo", "w")
 write.dcf(list(Description = 'what a fat goat .haha'),
@@ -471,19 +478,22 @@ close(zz)
 ## was " .haha" (not according to DCF standard)
 
 
-## Pdf() with CIDfonts active -- they need MBCS to be supported
-pdf(family="Japan1") # << for CIDfonts, pd->fonts is NULL
+## pdf() with CIDfonts active -- they need MBCS to be supported
+pdf(file = "testCID.pdf", family="Japan1") # << for CIDfonts, pd->fonts is NULL
 try({
     plot(1,1,pch="", axes=FALSE)
     text(1,1,"F.1", family="Helvetica")
 })
 dev.off()
+unlink("testCID.pdf")
 ## text() seg.faulted up to 2.7.2 (and early 2.8.0-alpha)
 
+
 ## PS mixing CIDfonts and Type1 - reverse case
-postscript(family="Helvetica")
+postscript(file = "testCID.ps", family="Helvetica")
 plot(1,1,pch="", axes=FALSE)
 try(text(1,1,"A",family="Japan1"))
+unlink("testCID.ps")
 ## error instead of seg.fault
 
 
@@ -792,7 +802,8 @@ stopifnot(inherits(con, "try-error") && nopen == nrow(showConnections()))
 
 ## PR#13574
 x <- 1:11; y <- c(6:1, 7, 11:8)
-stopifnot(all.equal(cor.test(x, y, method="spearman", alternative="greater")$p.value, cor.test(x, -y, method="spearman", alternative="less")$p.value))
+stopifnot(all.equal(cor.test(x, y, method="spearman", alternative="greater")$p.value,
+                    cor.test(x, -y, method="spearman", alternative="less")$p.value))
 ## marginally different < 2.9.0 patched
 
 
@@ -1035,6 +1046,7 @@ unlink("test.dat")
 stopifnot(identical(z, data.frame("B1.B2"="B3")))
 ## Left part of header to be read as data in R < 2.11.0
 
+
 ## switch() with  empty  '...'
 stopifnot(is.null(switch("A")),
 	  is.null(switch(1)), is.null(switch(3L)))
@@ -1333,7 +1345,8 @@ z2 <- quantile(x, type = 6, probs = c(.5, 0))
 stopifnot(z1 == rev(z2))
 ## differed in 2.11.x
 
-## backspline() with decreasing knot locations
+
+## backSpline() with decreasing knot locations
 require(splines)
 d1 <- c(616.1, 570.1, 523.7, 477.3, 431.3, 386.2, 342.4, 300.4, 260.4,
         222.7, 187.8, 155.7, 126.7, 100.8,  78.1,  58.6,  42.2,  28.7,
@@ -1411,8 +1424,8 @@ stopifnot(identical(attributes(!M), attributes(M)))
 
 
 ## Preserve intercepts in drop.terms
-tt <- terms(~a+b-1)
-tt2 <- terms(~b-1)
+tt <- terms(~ a + b - 1)
+tt2 <- terms(~ b - 1)
 stopifnot(identical(drop.terms(tt, 1), tt2))
 stopifnot(identical(tt[2], tt2))
 stopifnot(identical(tt[1:2], tt))
@@ -1425,17 +1438,10 @@ stopifnot(ncol(mds$points) == 11L)
 ## Used negative eigenvalues in 2.12.0
 
 
-## Sweave() comments with  keep.source=TRUE
-utils::Sweave(file.path(Sys.getenv("SRCDIR"), "keepsource.Rnw"))
-texl <- readLines("keepsource.tex")
-comml <- grep("##", texl, value=TRUE)
-stopifnot(length(comml) == 2,
-	  grepl("initial comment line", comml[1]),
-	  grepl("last comment", comml[2]))
-## the first was lost in 2.12.0;  the last in most/all previous versions of R
+## Sweave regression test moved to utils/tests.
 
 
-## mapply() & sapply() should not wrongly simplify e.g. for "call":
+## mapply() & sapply() should not simplify e.g. for "call":
 f2 <- function(i,j) call(':',i,j)
 stopifnot(identical(2:3,
 		    dim(sapply(1:3, function(i) list(0, 1:i)))),
@@ -1445,17 +1451,17 @@ stopifnot(identical(2:3,
 
 
 ## 'sep' in reshape() (PR#14335)
-test <- data.frame(x=rnorm(100), y=rnorm(100), famid=rep(1:50, each=2),
-                   time=rep(1:2, 50))
+test <- data.frame(x = rnorm(100), y = rnorm(100), famid = rep(1:50, each=2),
+                   time = rep(1:2, 50))
 
-wide <- reshape(data=test, v.names=c("x", "y"), idvar="famid",
-                timevar="time", sep="", direction="wide")
+wide <- reshape(data = test, v.names = c("x", "y"), idvar = "famid",
+                timevar = "time", sep = "", direction = "wide")
 stopifnot(identical(names(wide), c("famid", "x1", "y1", "x2", "y2")))
 ## was c("famid", "x.1", "y.1", "x.2", "y.2") in R <= 2.12.0
 
 
 ## PR#14438
-X <- matrix(0+1:10, ncol=2)[, c(1,1,2,2)]
+X <- matrix(0+1:10, ncol = 2)[, c(1,1,2,2)]
 colnames(X) <- c("X1","Dup1", "X2", "Dup2")
 X2 <- qr.X(qr(X))
 X2
@@ -1549,8 +1555,8 @@ unlink(tf)
 
 
 ## NA_complex_ in prettyNum()
-format(c(pi+0i, NA),   drop0=TRUE)
-prettyNum(NA_complex_, drop0=TRUE)
+format(c(pi+0i, NA),   drop0 = TRUE)
+prettyNum(NA_complex_, drop0 = TRUE)
 ## gave errors in R < 2.12.2
 
 
@@ -1585,7 +1591,7 @@ dfA <- data.frame(A=1:2, B=3:4, row.names=letters[1:2])
 dfB <- dfA[2:1,]
 res <- try(data.frame(dfA, dfA[2:1,], check.rows=TRUE))
 stopifnot(inherits(res, "try-error"))
-## worked in 2.12.2.
+## 'worked' in 2.12.2.
 
 
 ## uniroot(f,..) when f(.) == -Inf :
@@ -1606,4 +1612,54 @@ d <- dist(x)
 as.matrix(d)
 ## Threw an error < 2.13.0
 
+
+## smooth.spline with data with a very small range.  (PR#14552)
+dt <- seq(as.POSIXct("2011-01-01"), as.POSIXct("2011-01-01 10:00:00"), by="min")
+x <- as.double(dt)
+y <- sin(seq_along(x) * 3 * pi/180)
+s <- smooth.spline(x, y)
+stopifnot(length(s$x) == length(x))
+## Chose 5 distinct values of x in 2.13.0
+
+
+## readBin on a raw connection
+rawcon <- rawConnection(as.raw(101:110))
+res <- readBin(rawcon, what="integer", size=1, n=4)
+close(rawcon)
+stopifnot(identical(res, 101:104))
+## read the same value repeatedly in 2.13.0
+
+
+## Types of closure bodies
+fun <- eval(substitute(function() x, list(x = environment())))
+body(fun)
+# an external pointer
+y <- file(""); z <- attr(y, "conn_id"); close(y)
+fun <- eval(substitute(function() x, list(x = z)))
+body(fun)
+## not allowed in R < 2.14.0.
+
+
+## Corner cases for signif() and round()
+x <- pi^(-6:6)
+stopifnot(identical(signif(x, -Inf), signif(x, 1L))) # zero in R < 2.14.0
+stopifnot(identical(round(x, -Inf), rep(0, length(x)))) # NAs in R < 2.14.0
+##
+
+
+## (un)stack with character columns
+DF <- data.frame(a = letters[1:3], b = letters[4:6], stringsAsFactors = FALSE)
+DF2 <- stack(DF)
+stopifnot(class(DF2$values) == "character") # was factor
+DF3 <- unstack(DF2) # contained factors
+stopifnot(all(sapply(DF3, class) == "character"))
+DF4 <- stack(DF[1])
+stopifnot(identical(unstack(DF4), DF[1])) # was a list
+## issues in R < 2.14.0
+
+
+## PR#14710 (an instance of PR#8528)
+stopifnot(!is.na(qchisq(p=0.025, df=0.00991)))
+## NaN in 2.13.2
+
 proc.time()
